/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the "Elastic License
 * 2.0", the "GNU Affero General Public License v3.0 only", and the "Server Side
 * Public License v 1"; you may not use this file except in compliance with, at
 * your election, the "Elastic License 2.0", the "GNU Affero General Public
 * License v3.0 only", or the "Server Side Public License, v 1".
 */

import { i18n } from '@kbn/i18n';
import { prepareLogTable, validateAccessor } from '@kbn/visualizations-plugin/common/utils';
import { GaugeExpressionFunctionDefinition, GaugeRenderProps } from '../types';
import {
  EXPRESSION_GAUGE_NAME,
  GaugeColorModes,
  GaugeLabelMajorModes,
  GaugeShapes,
  GaugeTicksPositions,
} from '../constants';

const strings = {
  getMetricHelp: () =>
    i18n.translate('expressionGauge.logDatatable.metric', {
      defaultMessage: 'Metric',
    }),
  getMinHelp: () =>
    i18n.translate('expressionGauge.logDatatable.min', {
      defaultMessage: 'Min',
    }),
  getMaxHelp: () =>
    i18n.translate('expressionGauge.logDatatable.max', {
      defaultMessage: 'Max',
    }),
  getGoalHelp: () =>
    i18n.translate('expressionGauge.logDatatable.goal', {
      defaultMessage: 'Goal',
    }),
};

export const gaugeFunction = (): GaugeExpressionFunctionDefinition => ({
  name: EXPRESSION_GAUGE_NAME,
  type: 'render',
  inputTypes: ['datatable'],
  help: i18n.translate('expressionGauge.functions.gauge.help', {
    defaultMessage: 'Gauge visualization',
  }),
  args: {
    shape: {
      types: ['string'],
      options: [
        GaugeShapes.HORIZONTAL_BULLET,
        GaugeShapes.VERTICAL_BULLET,
        GaugeShapes.SEMI_CIRCLE,
        GaugeShapes.ARC,
        GaugeShapes.CIRCLE,
      ],
      help: i18n.translate('expressionGauge.functions.gauge.args.shape.help', {
        defaultMessage: 'Type of gauge chart',
      }),
      required: true,
      strict: true,
    },
    metric: {
      types: ['string', 'vis_dimension'],
      help: i18n.translate('expressionGauge.functions.gauge.args.metric.help', {
        defaultMessage: 'Current value',
      }),
    },
    min: {
      types: ['string', 'vis_dimension'],
      help: i18n.translate('expressionGauge.functions.gauge.args.min.help', {
        defaultMessage: 'Minimum value',
      }),
    },
    max: {
      types: ['string', 'vis_dimension'],
      help: i18n.translate('expressionGauge.functions.gauge.args.max.help', {
        defaultMessage: 'Maximum value',
      }),
    },
    goal: {
      types: ['string', 'vis_dimension'],
      help: i18n.translate('expressionGauge.functions.gauge.args.goal.help', {
        defaultMessage: 'Goal Value',
      }),
    },
    colorMode: {
      types: ['string'],
      default: GaugeColorModes.NONE,
      options: [GaugeColorModes.NONE, GaugeColorModes.PALETTE],
      help: i18n.translate('expressionGauge.functions.gauge.args.colorMode.help', {
        defaultMessage: 'If set to palette, the palette colors will be applied to the bands',
      }),
      strict: true,
    },
    palette: {
      types: ['palette'],
      help: i18n.translate('expressionGauge.functions.gauge.args.palette.help', {
        defaultMessage: 'Provides colors for the values',
      }),
    },
    ticksPosition: {
      types: ['string'],
      default: GaugeTicksPositions.AUTO,
      options: [GaugeTicksPositions.HIDDEN, GaugeTicksPositions.AUTO, GaugeTicksPositions.BANDS],
      help: i18n.translate('expressionGauge.functions.gauge.args.ticksPosition.help', {
        defaultMessage: 'Specifies the placement of ticks',
      }),
      strict: true,
    },
    labelMajor: {
      types: ['string'],
      help: i18n.translate('expressionGauge.functions.gauge.args.labelMajor.help', {
        defaultMessage: 'Specifies the labelMajor of the gauge chart displayed inside the chart.',
      }),
    },
    labelMajorMode: {
      types: ['string'],
      options: [GaugeLabelMajorModes.NONE, GaugeLabelMajorModes.AUTO, GaugeLabelMajorModes.CUSTOM],
      help: i18n.translate('expressionGauge.functions.gauge.args.labelMajorMode.help', {
        defaultMessage: 'Specifies the mode of labelMajor',
      }),
      default: GaugeLabelMajorModes.AUTO,
      strict: true,
    },
    labelMinor: {
      types: ['string'],
      help: i18n.translate('expressionGauge.functions.gauge.args.labelMinor.help', {
        defaultMessage: 'Specifies the labelMinor of the gauge chart',
      }),
    },
    centralMajor: {
      deprecated: true,
      types: ['string'],
      help: i18n.translate('expressionGauge.functions.gauge.args.centralMajor.help', {
        defaultMessage: 'Specifies the centralMajor of the gauge chart displayed inside the chart.',
      }),
    },
    centralMajorMode: {
      deprecated: true,
      types: ['string'],
      options: [GaugeLabelMajorModes.NONE, GaugeLabelMajorModes.AUTO, GaugeLabelMajorModes.CUSTOM],
      help: i18n.translate('expressionGauge.functions.gauge.args.centralMajorMode.help', {
        defaultMessage: 'Specifies the mode of centralMajor',
      }),
      strict: true,
    },
    percentageMode: {
      deprecated: true, // used only in legacy gauge, consider it as @deprecated
      types: ['boolean'],
      default: false,
      help: i18n.translate('expressionGauge.functions.gauge.percentageMode.help', {
        defaultMessage: 'Enables relative precentage mode',
      }),
    },
    respectRanges: {
      types: ['boolean'],
      default: false,
      help: i18n.translate('expressionGauge.functions.gauge.respectRanges.help', {
        defaultMessage: 'Respect max and min values from ranges',
      }),
    },
    commonLabel: {
      types: ['string'],
      help: i18n.translate('expressionGauge.functions.gauge.args.commonLabel.help', {
        defaultMessage: 'Specifies the common label outside the chart',
      }),
    },
    ariaLabel: {
      types: ['string'],
      help: i18n.translate('expressionGauge.functions.gaugeChart.config.ariaLabel.help', {
        defaultMessage: 'Specifies the aria label of the gauge chart',
      }),
    },
  },
  fn(data, args, handlers) {
    validateAccessor(args.metric, data.columns);
    validateAccessor(args.min, data.columns);
    validateAccessor(args.max, data.columns);
    validateAccessor(args.goal, data.columns);

    const { metric, min, max, goal } = args;

    if (handlers?.inspectorAdapters?.tables) {
      handlers.inspectorAdapters.tables.reset();
      handlers.inspectorAdapters.tables.allowCsvExport = true;

      const logTable = prepareLogTable(
        data,
        [
          [metric ? [metric] : undefined, strings.getMetricHelp()],
          [min ? [min] : undefined, strings.getMinHelp()],
          [max ? [max] : undefined, strings.getMaxHelp()],
          [goal ? [goal] : undefined, strings.getGoalHelp()],
        ],
        true
      );

      handlers.inspectorAdapters.tables.logDatatable('default', logTable);
    }

    return {
      type: 'render',
      as: EXPRESSION_GAUGE_NAME,
      value: {
        data,
        args: {
          ...args,
          ariaLabel:
            args.ariaLabel ??
            (handlers.variables?.embeddableTitle as string) ??
            handlers.getExecutionContext?.()?.description,
        },
        canNavigateToLens: Boolean(handlers?.variables?.canNavigateToLens),
        overrides: handlers.variables?.overrides as GaugeRenderProps['overrides'],
      },
    };
  },
});
