/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the "Elastic License
 * 2.0", the "GNU Affero General Public License v3.0 only", and the "Server Side
 * Public License v 1"; you may not use this file except in compliance with, at
 * your election, the "Elastic License 2.0", the "GNU Affero General Public
 * License v3.0 only", or the "Server Side Public License, v 1".
 */

import { dropRight, last } from 'lodash';
import { getTopHitMetricAgg } from './top_hit';
import { AggConfigs } from '../agg_configs';
import { mockAggTypesRegistry } from '../test_helpers';
import { IMetricAggConfig } from './metric_agg_type';
import { KBN_FIELD_TYPES } from '../../..';
import * as tabifyModule from '@kbn/data-service/src/search/tabify/tabify_docs';

describe('Top hit metric', () => {
  let aggDsl: Record<string, any>;
  let aggConfig: IMetricAggConfig;

  const init = ({
    fieldName = 'field',
    sortOrder = 'desc',
    aggregate = 'concat',
    readFromDocValues = false,
    fieldType = KBN_FIELD_TYPES.NUMBER,
    size = 1,
  }: any) => {
    const typesRegistry = mockAggTypesRegistry();
    const field = {
      name: fieldName,
      displayName: fieldName,
      type: fieldType,
      readFromDocValues,
      format: {
        type: {
          id: 'bytes',
        },
      },
    };

    const params = {
      size,
      field: fieldName,
      sortField: field,
      sortOrder: {
        value: sortOrder,
      },
      aggregate: {
        value: aggregate,
      },
    };

    const indexPattern = {
      id: '1234',
      title: 'logstash-*',
      fields: {
        getByName: () => field,
        filter: () => [field],
      },
      flattenHit: jest.fn((x) => x!._source),
    } as any;

    const aggConfigs = new AggConfigs(
      indexPattern,
      [
        {
          id: '1',
          type: 'top_hits',
          schema: 'metric',
          params,
        },
      ],
      { typesRegistry },
      jest.fn()
    );

    // Grab the aggConfig off the vis (we don't actually use the vis for anything else)
    aggConfig = aggConfigs.aggs[0] as IMetricAggConfig;
    aggDsl = aggConfig.toDsl(aggConfigs);
  };

  const flattenSpy = jest.spyOn(tabifyModule, 'flattenHit');
  beforeEach(() => {
    jest.clearAllMocks();
  });

  it('should return a label prefixed with Last if sorting in descending order', () => {
    init({ fieldName: 'bytes' });
    expect(getTopHitMetricAgg().makeLabel(aggConfig)).toEqual('Last bytes');
  });

  it('should return a label prefixed with First if sorting in ascending order', () => {
    init({
      fieldName: 'bytes',
      sortOrder: 'asc',
    });
    expect(getTopHitMetricAgg().makeLabel(aggConfig)).toEqual('First bytes');
  });

  it('produces the expected expression ast', () => {
    init({ fieldName: 'machine.os' });
    expect(aggConfig.toExpressionAst()).toMatchInlineSnapshot(`
      Object {
        "chain": Array [
          Object {
            "arguments": Object {
              "aggregate": Array [
                "concat",
              ],
              "enabled": Array [
                true,
              ],
              "field": Array [
                "machine.os",
              ],
              "id": Array [
                "1",
              ],
              "schema": Array [
                "metric",
              ],
              "size": Array [
                1,
              ],
              "sortField": Array [
                "machine.os",
              ],
              "sortOrder": Array [
                "desc",
              ],
            },
            "function": "aggTopHit",
            "type": "function",
          },
        ],
        "type": "expression",
      }
    `);
  });

  it('should request the _source field', () => {
    init({ fieldName: '_source' });
    expect(aggDsl.top_hits._source).toBe(true);
    expect(aggDsl.top_hits.fields).toBeUndefined();
  });

  it('requests fields for non-text aggregatable fields', () => {
    init({ fieldName: 'bytes', readFromDocValues: true });
    expect(aggDsl.top_hits._source).toBe(false);
    expect(aggDsl.top_hits.fields).toEqual([{ field: 'bytes' }]);
  });

  it('requests fields for date aggregatable fields', () => {
    init({ fieldName: '@timestamp', readFromDocValues: true, fieldType: KBN_FIELD_TYPES.DATE });

    expect(aggDsl.top_hits._source).toBe(false);
    expect(aggDsl.top_hits.fields).toEqual([{ field: '@timestamp', format: 'date_time' }]);
  });

  it('requests fields for aggregatable text fields', () => {
    init({ fieldName: 'machine.os' });
    expect(aggDsl.top_hits._source).toBe(false);
    expect(aggDsl.top_hits.fields).toEqual([{ field: 'machine.os' }]);
  });

  describe('try to get the value from the top hit', () => {
    it('should return null if there is no hit', () => {
      const bucket = {
        '1': {
          hits: {
            hits: [],
          },
        },
      };

      init({ fieldName: '@tags' });
      expect(getTopHitMetricAgg().getValue(aggConfig, bucket)).toBe(null);
    });
    //
    it('should return undefined if the field does not appear in the source', () => {
      const bucket = {
        '1': {
          hits: {
            hits: [
              {
                _source: {
                  bytes: 123,
                },
              },
            ],
          },
        },
      };

      init({ fieldName: '@tags' });
      expect(getTopHitMetricAgg().getValue(aggConfig, bucket)).toBe(undefined);
    });

    it('should return the field value from the top hit', () => {
      const bucket = {
        '1': {
          hits: {
            hits: [
              {
                _source: {
                  '@tags': 'aaa',
                },
              },
            ],
          },
        },
      };

      init({ fieldName: '@tags' });
      expect(getTopHitMetricAgg().getValue(aggConfig, bucket)).toBe('aaa');
      expect(flattenSpy).toHaveLastReturnedWith({ '@tags': 'aaa' });
    });

    it('should return the object if the field value is an object', () => {
      const bucket = {
        '1': {
          hits: {
            hits: [
              {
                _source: {
                  '@tags': {
                    label: 'aaa',
                  },
                },
              },
            ],
          },
        },
      };

      init({ fieldName: '@tags' });

      expect(getTopHitMetricAgg().getValue(aggConfig, bucket)).toEqual({
        label: 'aaa',
      });
      expect(flattenSpy).toHaveLastReturnedWith({ '@tags': { label: 'aaa' } });
    });

    it('should return an array if the field has more than one values', () => {
      const bucket = {
        '1': {
          hits: {
            hits: [
              {
                _source: {
                  '@tags': ['aaa', 'bbb'],
                },
              },
            ],
          },
        },
      };

      init({ fieldName: '@tags' });
      expect(getTopHitMetricAgg().getValue(aggConfig, bucket)).toEqual(['aaa', 'bbb']);
      expect(flattenSpy).toHaveLastReturnedWith({ '@tags': ['aaa', 'bbb'] });
    });

    it('should return undefined if the field is not in the source nor in the doc_values field', () => {
      const bucket = {
        '1': {
          hits: {
            hits: [
              {
                _source: {
                  bytes: 12345,
                },
                fields: {
                  bytes: 12345,
                },
              },
            ],
          },
        },
      };

      init({ fieldName: 'machine.os.raw', readFromDocValues: true });
      expect(getTopHitMetricAgg().getValue(aggConfig, bucket)).toBe(undefined);
    });

    describe('Multivalued field and first/last X docs', () => {
      it('should return a label prefixed with Last X docs if sorting in descending order', () => {
        init({
          fieldName: 'bytes',
          size: 2,
        });
        expect(getTopHitMetricAgg().makeLabel(aggConfig)).toEqual('Last 2 bytes');
      });

      it('should return a label prefixed with First X docs if sorting in ascending order', () => {
        init({
          fieldName: 'bytes',
          size: 2,
          sortOrder: 'asc',
        });
        expect(getTopHitMetricAgg().makeLabel(aggConfig)).toEqual('First 2 bytes');
      });

      [
        {
          description: 'concat values with a comma',
          type: 'concat',
          data: [1, 2, 3],
          result: [1, 2, 3],
        },
        {
          description: 'sum up the values',
          type: 'sum',
          data: [1, 2, 3],
          result: 6,
        },
        {
          description: 'take the minimum value',
          type: 'min',
          data: [1, 2, 3],
          result: 1,
        },
        {
          description: 'take the maximum value',
          type: 'max',
          data: [1, 2, 3],
          result: 3,
        },
        {
          description: 'take the average value',
          type: 'average',
          data: [1, 2, 3],
          result: 2,
        },
        {
          description: 'support null/undefined',
          type: 'min',
          data: [undefined, null],
          result: null,
        },
        {
          description: 'support null/undefined',
          type: 'max',
          data: [undefined, null],
          result: null,
        },
        {
          description: 'support null/undefined',
          type: 'sum',
          data: [undefined, null],
          result: null,
        },
        {
          description: 'support null/undefined',
          type: 'average',
          data: [undefined, null],
          result: null,
        },
      ].forEach((agg) => {
        it(`should return the result of the ${agg.type} aggregation over the last doc - ${agg.description}`, () => {
          const bucket = {
            '1': {
              hits: {
                hits: [
                  {
                    _source: {
                      bytes: agg.data,
                    },
                  },
                ],
              },
            },
          };

          init({ fieldName: 'bytes', aggregate: agg.type });
          expect(getTopHitMetricAgg().getValue(aggConfig, bucket)).toEqual(agg.result);
        });

        it(`should return the result of the ${agg.type} aggregation over the last X docs - ${agg.description}`, () => {
          const bucket = {
            '1': {
              hits: {
                hits: [
                  {
                    _source: {
                      bytes: dropRight(agg.data, 1),
                    },
                  },
                  {
                    _source: {
                      bytes: last(agg.data),
                    },
                  },
                ],
              },
            },
          };

          init({ fieldName: 'bytes', aggregate: agg.type });
          expect(getTopHitMetricAgg().getValue(aggConfig, bucket)).toEqual(agg.result);
        });
      });
    });
    it('returns phrase filter', () => {
      expect(getTopHitMetricAgg().createFilter!(aggConfig, '10').query.match_phrase).toEqual({
        bytes: 10,
      });
    });
  });
});
